import test from 'ava';
import keycode from 'keycode';
import { contains, reducerCaseTest } from 'graftss-redux-utils';
import { merge } from 'ramda';

import { creators } from '../src/KeystrokeHistory/actions';
import reducer from '../src/KeystrokeHistory/reducer';
import { initialState } from '../src/KeystrokeHistory/state';

const {
  clearValue,
  keyDown,
  keyUp,
  setValue,
} = creators;

const withInitialState = merge(initialState);

const reducerCaseMacro = (t, caseData, compare = t => ::t.deepEqual) => (
  reducerCaseTest(compare(t), reducer, caseData)
);

const containsCompare = t => (a, b) => t.true(contains(a, b));

const keydownMacro = (key) => {
  const code = keycode(key);

  test(`key down - "${key}" key`, reducerCaseMacro, {
    initialState,
    actions: [keyDown(key)],
    finalState: {
      keys: { [code]: 0 },
      history: [{ keycode: code }],
    },
  }, containsCompare);
};

['Shift', 'Ctrl', 'Alt'].forEach(keydownMacro);

test('clear value', reducerCaseMacro, {
  initialState,
  actions: [clearValue()],
  finalState: withInitialState({ value: '' }),
});

test('set value', reducerCaseMacro, {
  initialState,
  actions: [setValue('123')],
  finalState: withInitialState({ value: '123' }),
});

const finalKeydownState = withInitialState({
  keys: { [keycode('s')]: 0 },
  history: [{ keycode: keycode('s') }],
});

test('key down - single key', reducerCaseMacro, {
  initialState,
  actions: [keyDown('s')],
  finalState: finalKeydownState,
}, containsCompare);

test('key down - case insensitivity', reducerCaseMacro, {
  initialState,
  actions: [keyDown('S')],
  finalState: finalKeydownState,
}, containsCompare);

test('key down - unresolved keydown', reducerCaseMacro, {
  initialState,
  actions: [keyDown('s'), keyDown('s')],
  finalState: withInitialState({
    keys: { [keycode('s')]: 1 },
    history: [
      { keycode: keycode('s'), unresolved: true },
      { keycode: keycode('s'), unresolved: undefined },
    ],
  }),
}, containsCompare);

test('key down - repeated event with unrelated key down', reducerCaseMacro, {
  initialState,
  actions: [keyDown('s'), keyDown('t'), keyDown('s')],
  finalState: {
    keys: { [keycode('t')]: 1, [keycode('s')]: 2 },
    history: [
      { keycode: keycode('s'), unresolved: true },
      { keycode: keycode('t'), unresolved: undefined },
      { keycode: keycode('s'), unresolved: undefined },
    ],
  },
}, containsCompare);

test('key up - ignore keys that aren\'t down', reducerCaseMacro, {
  initialState,
  actions: [keyUp('s')],
  finalState: initialState,
});

test('key up - handle keys that are down', reducerCaseMacro, {
  initialState,
  actions: [keyDown('s'), keyUp('s')],
  finalState: withInitialState({
    history: [{ keycode: keycode('s') }],
  }),
}, containsCompare);

test('key up - case insensitivity', reducerCaseMacro, {
  initialState,
  actions: [keyDown('s'), keyUp('S')],
  finalState: withInitialState({
    history: [{ keycode: keycode('s') }],
  }),
}, containsCompare);
